
import os

from ai.controller.gemini_controller import GeminiController
from ai.controller.mistral_controller import MistralController
from ai.llm.local.llamacpp import LlamaCpp
from ai.core.ai_setting import AiSetting, AiQuery
from ai.core.ai_source_type import AiSourceType
from media.liz_media import LizMedia
from model.operation import Operation
from util.jsonUtils import JsonUtils
from util.pylizdir import PylizDir


class AiRunner:

    def __init__(self, pyliz_dir: PylizDir, query: AiQuery):
        self.query = query
        self.pyliz_dir = pyliz_dir
        self.folder_ai = self.pyliz_dir.get_folder_path("ai")
        self.folder_logs = self.pyliz_dir.get_folder_path("logs")
        self.model_folder = self.pyliz_dir.get_folder_path("models")

    def __handle_mistral(self) -> Operation[str]:
        controller = MistralController(self.query.setting.api_key)
        return controller.run(self.query)

    def __handle_git_llama_cpp(self) -> Operation[str]:
        folder = os.path.join(self.folder_ai, "llama.cpp")
        logs = os.path.join(self.folder_logs, "llama.cpp")
        llama_cpp = LlamaCpp(folder, self.model_folder, logs)
        pass

    def __handle_gemini(self):
        controller = GeminiController(self.query.setting.api_key)
        return controller.run(self.query)


    def run(self) -> Operation[str]:
        if self.query.setting.source_type == AiSourceType.API_MISTRAL:
            return self.__handle_mistral()
        if self.query.setting.source_type == AiSourceType.LOCAL_LLAMACPP:
            return self.__handle_git_llama_cpp()
        if self.query.setting.source_type == AiSourceType.API_GEMINI:
            return self.__handle_gemini()
        raise NotImplementedError("Source type not implemented yet in AiRunner")




# class AiInputs:
#
#     def __init__(self,
#                  file_path: str | None = None,
#                  prompt: str | None = None,
#                 ):
#         self.file_path = file_path
#         self.prompt = prompt



import json
import time
from typing import Callable

from loguru import logger

from ai.core.ai_setting import AiSetting, AiQuery
from ai.prompt.ai_prompts import AiPrompt
from ai.util.ai_runner import AiRunner
from media.liz_media import LizMedia
from model.operation import Operation
from util import fileutils
from util.jsonUtils import JsonUtils
from util.pylizdir import PylizDir
from enum import Enum


class PixelRunnerMethod(Enum):
    DOUBLE_QUERY_WITH_TEXT_GEN = "DOUBLE_QUERY_WITH_TEXT_GEN"
    SINGLE_QUERY_ONLY_VISION = "SINGLE_QUERY_ONLY_VISION"


class AiPixelRunner:

    def __init__(
        self,
        pyliz_dir: PylizDir,
        image_method: PixelRunnerMethod,
        ai_image_setting: AiSetting,
        ai_text_setting: AiSetting | None = None
    ):
        self.pyliz_dir = pyliz_dir
        self.image_method = image_method
        self.ai_image_setting = ai_image_setting
        self.ai_text_setting = ai_text_setting

        if image_method == PixelRunnerMethod.DOUBLE_QUERY_WITH_TEXT_GEN and ai_text_setting is None:
            raise ValueError("ai_text_setting is required for DOUBLE_QUERY_WITH_TEXT_GEN in AiPixelRunner")


    def __exec_image_query(self, media_path: str, prompt: str) -> str:
        query = AiQuery(self.ai_image_setting, prompt, media_path)
        ai_image_result = AiRunner(self.pyliz_dir, query).run()
        if not ai_image_result.status:
            raise ValueError(ai_image_result.error)
        logger.info(f"RunForMedia (image) result = {ai_image_result}")
        return ai_image_result.payload

    def __exec_text_query(self, prompt: str) -> str:
        ai_text_result = AiRunner(self.pyliz_dir, AiQuery(self.ai_text_setting, prompt)).run()
        logger.info(f"RunForMedia (text) result = {ai_text_result}")
        if not ai_text_result.status:
            raise ValueError(ai_text_result.error)
        return ai_text_result.payload

    def __handle_image_with_text_gen(
            self,
            media_path: str,
            on_log: Callable[[str], None] = lambda x: None
    ) -> Operation[LizMedia]:
        try:
            # Image query
            on_log("Running image query")
            ai_image_result = self.__exec_image_query(media_path, AiPrompt.IMAGE_VISION_DETAILED_1.value)
            # Text query
            on_log("Generating text")
            ai_text_result = self.__exec_text_query(AiPrompt.TEXT_EXTRACT_FROM_VISION_1.value + ai_image_result)
            # Media creation
            on_log("Generating object")
            time.sleep(0.5)
            media = LizMedia(media_path)
            # Extract ai info from json
            on_log("Validating ai results")
            json_result_text = ai_text_result
            if not JsonUtils.is_valid_json(json_result_text):
                raise ValueError("Ai returned invalid json")
            if not JsonUtils.has_keys(json_result_text, ["text", "tags", "filename"]):
                raise ValueError("Ai returned invalid json keys")
            data = json.loads(json_result_text)
            media.ai_ocr_text = data['text']
            media.ai_description = ai_image_result
            media.ai_tags = data['tags']
            media.ai_file_name = data['filename']
            media.ai_scanned = True
            time.sleep(0.5)
            on_log("Completed")
            return Operation(status=True, payload=media)
        except Exception as e:
            return Operation(status=False, error=str(e))


    def __handle_image_only_vision(self, media_path: str, on_log: Callable[[str], None] = lambda x: None) -> Operation[LizMedia]:
        try:
            # Image query
            on_log("Running image query")
            ai_image_result = self.__exec_image_query(media_path, AiPrompt.IMAGE_VISION_JSON.value)
            logger.info(f"RunForMedia (image) result = {ai_image_result}")
            raise NotImplementedError("Image only vision not implemented yet in AiPixelRunner")
        except Exception as e:
            return Operation(status=False, error=str(e))



    def __run_image(
            self,
            media_path: str,
            on_log: Callable[[str], None] = lambda x: None
    ) -> Operation[LizMedia]:
        if self.image_method == PixelRunnerMethod.DOUBLE_QUERY_WITH_TEXT_GEN:
            return self.__handle_image_with_text_gen(media_path, on_log)
        elif self.image_method == PixelRunnerMethod.SINGLE_QUERY_ONLY_VISION:
            return self.__handle_image_only_vision(media_path, on_log)
        else:
            raise ValueError("Unsupported image_method in AiPixelRunner")

    def __run_video(self, media_path: str, on_log: Callable[[str], None] = lambda x: None) -> Operation[LizMedia]:
        raise NotImplementedError("Video not implemented yet in AiPixelRunner")


    def scan(
            self,
            media_path: str,
            on_log: Callable[[str], None] = lambda x: None
    ) -> Operation[LizMedia]:
        if fileutils.is_image_file(media_path):
            return self.__run_image(media_path, on_log)
        elif fileutils.is_video_file(media_path):
            return self.__run_video(media_path, on_log)
        else:
            raise ValueError("Unsupported file type in AiPixelRunner")






import os
import time

import google.generativeai as genai
from loguru import logger

from ai.core.ai_setting import AiSetting, AiQuery
from model.operation import Operation
from util import fileutils


class GeminiController:

    def __init__(self, key: str):
        genai.configure(api_key=key)

    @staticmethod
    def __upload(self, path: str, file_name: str):
        uri = sample_file = genai.upload_file(path=path, display_name=file_name)
        logger.info(f"Uploaded file to Google temp cache: {sample_file}")
        return uri

    @staticmethod
    def __verify_loaded_video(self, uri):
        # Check whether the file is ready to be used.
        while uri.state.name == "PROCESSING":
            logger.debug('.', end='')
            time.sleep(10)
            video_file = genai.get_file(uri.name)

        if uri.state.name == "FAILED":
            raise ValueError(uri.state.name)

    def scan_image(self, query: AiQuery) -> Operation[str]:
        try:
            path = query.payload_path
            file_name = os.path.basename(path)
            uri = self.__upload(path, file_name)
            model = genai.GenerativeModel(model_name=query.setting.source.model_name)
            response = model.generate_content([uri, query.prompt])
            genai.delete_file(uri.name)
            logger.info("result =" + response.text)
            return Operation(status=True, payload=response.text)
        except Exception as e:
            return Operation(status=False, error=str(e))

    def scan_video(self, query: AiQuery) -> Operation[str]:
        try:
            path = query.payload_path
            file_name = os.path.basename(path)
            uri = self.__upload(path, file_name)
            self.__verify_loaded_video(uri)
            model = genai.GenerativeModel(model_name=query.setting.source.model_name)
            response = model.generate_content([uri, query.prompt], request_options={"timeout": 600})
            logger.debug("result =" + response.text)
            genai.delete_file(uri.name)
            return Operation(status=True, payload=response.text)
        except Exception as e:
            return Operation(status=False, error=str(e))

    def run(self, query: AiQuery) -> Operation[str]:
        path = query.payload_path
        if fileutils.is_image_file(path):
            return self.scan_image(query)
        elif fileutils.is_video_file(path):
            return self.scan_video(query)







import os
import unittest

import rich

from ai.core.ai_model_list import AiModelList
from ai.core.ai_power import AiPower
from ai.core.ai_setting import AiSetting
from ai.core.ai_source_type import AiSourceType
from ai.llm.remote.service.lmstudioliz import LmStudioLiz
import sys
import os
from dotenv import load_dotenv

from ai.util.ai_pixel_runner import AiPixelRunner, PixelRunnerMethod
from ai.util.ai_runner import AiCustomRunner
from util import pylizLogging
from util.pylizdir import PylizDir

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))


class TestAiImage(unittest.TestCase):

    def setUp(self):
        load_dotenv()
        pylizLogging.enable_logging("DEBUG", None, True)
        print("Setting up test...")


    def test1(self):
        pyliz_dir = PylizDir(".pyliztest")
        image = os.getenv('LOCAL_IMAGE_FOR_TEST')
        api_key = os.getenv('MISTRAL_API_KEY')
        ai_image_setting = AiSetting(
            model=AiModelList.PIXSTRAL,
            source_type=AiSourceType.API_MISTRAL,
            power=AiPower.MEDIUM,
            api_key=api_key,
        )
        ai_text_setting = AiSetting(
            model=AiModelList.OPEN_MISTRAL,
            source_type=AiSourceType.API_MISTRAL,
            power=AiPower.LOW,
            api_key=api_key,
        )
        pixel_runner = AiPixelRunner(pyliz_dir, PixelRunnerMethod.DOUBLE_QUERY_WITH_TEXT_GEN, ai_image_setting, ai_text_setting)
        media = pixel_runner.scan(image)
        rich.print("----")
        rich.print(media.payload.ai_description)
        rich.print(media.payload.ai_file_name)
        rich.print("end")



if __name__ == "__main__":
    unittest.main()



import os
import unittest

from ai.llm.local.llamacpplib import LlamaCppLib
from util.pylizdir import PylizDir


class TestLlamaCPPLib(unittest.TestCase):

    def setUp(self):
        print("Setting up test...")



    def test1(self):
        dir = PylizDir(".medializ")
        LlamaCppLib.run_llama3("What is SpaceX ?", dir.get_path())


if __name__ == "__main__":
    unittest.main()


import os
import unittest

import rich

from ai.llm.local.llamacpp import LlamaCpp
from ai.core.ai_power import AiPower
from ai.prompt.ai_prompts import prompt_llava_json
from util.pylizdir import PylizDir


def log(message: str):
    rich.print(message)


def progress(percent: int):
    rich.print(f"Progress: {percent}%")


class TestLlamaCPP(unittest.TestCase):

    def setUp(self):
        print("Setting up test...")
        self.dir = PylizDir(".pyliztest")
        self.ai_folder = self.dir.add_folder("ai", "ai")
        self.model_folder = self.dir.add_folder("models", "models")
        self.log_folder = self.dir.add_folder("log", "log")
        self.path_install = os.path.join(self.ai_folder, "llama.cpp")
        self.path_models = os.path.join(self.model_folder, "llama.cpp")
        self.path_logs = os.path.join(self.log_folder, "llama.cpp")
        self.obj = LlamaCpp(self.path_install, self.path_models, self.path_logs)

    def test_install_llava(self):
        try:
            self.obj.install_llava(AiPower.LOW, log, progress)
        except Exception as e:
            self.fail(e)

    def test_run_llava(self):
        try:
            result = self.obj.run_llava(AiPower.LOW, "/Users/gabliz/Pictures/obama343434333.jpg", prompt_llava_json)
            print(result)
        except Exception as e:
            self.fail(e)


if __name__ == "__main__":
    unittest.main()


from abc import ABC, abstractmethod
from enum import Enum


class PromptType(Enum):
    TEXT_PROMPT = "text_prompt"
    IMAGE_PROMPT = "image_prompt"


class CustomPrompt(ABC):

    prompt_type = None

    @property
    @abstractmethod
    def text(self):
        pass


class PromptInfo:

    def __init__(self, text: str, type: PromptType):
        self.text = text
        self.type = type



import unittest

from ai.util.ai_runner import AiRunner
from ai.core.ai_model_list import AiModelList
from ai.core.ai_power import AiPower
from ai.prompt.ai_prompts import AiPrompt
from ai.core.ai_setting import AiSetting
from ai.core.ai_source_type import AiSourceType
import sys
import os
from dotenv import load_dotenv

from util.pylizdir import PylizDir

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))


class TestLmStudio(unittest.TestCase):

    def setUp(self):
        load_dotenv()
        print("Setting up test...")
        self.pyliz_dir = None


    def test1(self):
        self.pyliz_dir = PylizDir(".pyliztest")
        setting = AiSetting(
            model=AiModelList.GEMINI,
            source_type=AiSourceType.API_GEMINI,
            power=AiPower.LOW,
            payload_file_path=os.getenv('LOCAL_IMAGE_FOR_TEST'),
            api_key=os.getenv('GEMINI_API_KEY'),
            prompt="Analyze this image and tell me what you see."
        )
        result = AiRunner(self.pyliz_dir, setting).run()
        print("result status = " + str(result.status))
        print(result.payload)
        print("result error = " + result.error if result.error is not None else "No error")


    def test2(self):
        self.pyliz_dir = PylizDir(".pyliztest")
        setting = AiSetting(
            model=AiModelList.GEMINI,
            source_type=AiSourceType.API_GEMINI,
            power=AiPower.LOW,
            payload_file_path=os.getenv('LOCAL_VIDEO_FOR_TEST'),
            api_key=os.getenv('GEMINI_API_KEY'),
            prompt="Analyze this video and tell me what you see."
        )
        result = AiRunner(self.pyliz_dir, setting).run()
        print("result status = " + str(result.status))
        print(result.payload)
        print("result error = " + result.error if result.error is not None else "No error")






if __name__ == "__main__":
    unittest.main()